require_relative 'player' 
require_relative 'die' 
require_relative 'game_turn' 
require_relative 'treasure_trove' 
 
module StudioGame  
  class Game
    attr_reader :title
  
    def initialize(title)
      @title = title.capitalize
      @players = [] 	
    end
  
    def load_players(from_file)
      File.readlines(from_file).each do |line| 
	    add_player(Player.from_csv(line)) 
	  end 
    end 

    def save_high_scores(to_file='high_scores.txt') 
      File.open(to_file, 'w') do |file| 
	    file.puts "#{@title} High Scores:"
        @players.sort.each do |p| 
	      file.puts high_score_entry(p) 
	    end 	
	  end   
    end

    def high_score_entry(player)
      "#{player.name.ljust(20, '.')}#{player.score}"
    end 	
   
    def add_player(player)
      @players.push(player)
    end 
	
    def play(rounds)  
      puts "\nThere are #{@players.size} players in #{@title}:"
	  puts @players 
	
	  treasures = TreasureTrove:: TREASURES
      puts "\nThere are #{treasures.size} treasures to be found:" 
      treasures.each do |t| 
        puts "A #{t.name} is worth #{t.points} points."
	  end 
	
	  1.upto(rounds) do |round|
	    puts "\nRound #{round}:"
	    @players.each do |p|
          GameTurn.take_turn(p)		
	      puts p 
	    end 	
	  end 
    end
  
    def print_name_and_health(p) 
      puts "#{p.name} (#{p.health})"
    end 	

    def print_stats 
      strong_players, wimpy_players = @players.partition {|p| p.strong?}
	
      puts "\n#{@title} Statistics:" 
	
	  puts "\n#{strong_players.size} strong players:"
	  strong_players.each do |p| 
	    print_name_and_health(p)
	  end 
	
	  puts "\n#{wimpy_players.size} wimpy players:"
      wimpy_players.each do |p| 
        print_name_and_health(p) 
      end 
	
	  @players.each do |p| 
	    puts "\n#{p.name}'s point totals:"
	    p.each_found_treasure do |t| 
	      puts "#{t.points} total #{t.name} points"
	    end 	
	    puts "#{p.points} grand total points."
	  end  
      	
      puts "\n#{@title} High Scores:" 
      @players.sort.each do |p| 
        puts high_score_entry(p)
      end 	  
    end   	
  end
end 
  
  